import asyncio
import logging
from asyncio.tasks import Task
from contextlib import suppress
from inspect import isasyncgenfunction
from signal import SIGINT, SIGTERM
from typing import Any, AsyncIterator, Callable, Final, Mapping, Optional

__all__ = ('ContextFunction', 'Runner')

ContextFunction = Callable[..., AsyncIterator[None]]

logger = logging.getLogger('runner')


class Runner:

    def __init__(
        self, context_function: ContextFunction, debug: bool = False,
        **kwargs: Any
    ) -> None:
        if not isasyncgenfunction(context_function):
            raise RuntimeError('Argument is not async generator')
        self._context_function: Final[ContextFunction] = context_function
        self._kwargs: Final[Mapping[str, Any]] = kwargs
        self._debug: Final[bool] = debug
        self._wait_task: Optional[Task[None]] = None
        self._started: bool = False
        self._stopped: bool = False

    async def _run(self) -> None:
        assert not self._started
        assert not self._stopped
        self._started = True
        loop = asyncio.get_running_loop()
        loop.add_signal_handler(SIGINT, self._signal_handler, SIGINT.name)
        loop.add_signal_handler(SIGTERM, self._signal_handler, SIGTERM.name)
        logger.debug('Enter wait loop')
        iterator = self._context_function(self, **self._kwargs).__aiter__()
        await iterator.__anext__()
        self._wait_task = asyncio.create_task(self._wait())
        with suppress(asyncio.CancelledError):
            await self._wait_task
        logger.debug('Waiting finished')
        try:
            await iterator.__anext__()
        except StopAsyncIteration:
            pass
        else:
            raise RuntimeError(f'{iterator!r} has more than one \'yield\'')

    async def _wait(self) -> None:
        assert self._started
        assert not self._stopped
        while not self._stopped:
            await asyncio.sleep(3600)

    def _signal_handler(self, signal_name: str) -> None:
        logger.debug('Received signal %s', signal_name)
        self.stop()

    def stop(self) -> None:
        if not self._started:
            raise RuntimeError('Not started')
        if self._stopped:
            raise RuntimeError('Already stopped')
        assert self._wait_task is not None
        self._stopped = True
        self._wait_task.cancel()

    def run(self) -> None:
        if self._started:
            raise RuntimeError('Already started')
        asyncio.run(self._run(), debug=self._debug)



import re
from typing import Final, Tuple

import attr

from . import Bot, FilterProtocol
from .bot_update import BotUpdate
from .constants import ChatType, ContentType, UpdateType

__all__ = ('UpdateTypeFilter', 'StateFilter', 'CommandsFilter',
           'ContentTypeFilter', 'MessageTextFilter', 'CallbackQueryDataFilter',
           'PrivateChatFilter', 'GroupChatFilter', 'ORFilter', 'ANDFilter')


@attr.s(frozen=True, auto_attribs=True)
class UpdateTypeFilter:
    update_type: UpdateType

    async def check(self, _: Bot, update: BotUpdate) -> bool:
        return getattr(update, self.update_type.value) is not None


@attr.s(frozen=True, auto_attribs=True)
class StateFilter:
    state: str

    async def check(self, _: Bot, update: BotUpdate) -> bool:
        return update.state == self.state


@attr.s(frozen=True, auto_attribs=True)
class CommandsFilter:
    commands: Tuple[str, ...]

    async def check(self, _: Bot, update: BotUpdate) -> bool:
        if update.message is None or update.message.text is None:
            return False
        if any(update.message.text.startswith(f'/{command}')
               for command in self.commands):
            return True
        return False


@attr.s(frozen=True, auto_attribs=True)
class ContentTypeFilter:
    content_types: Tuple[ContentType, ...]

    async def check(self, _: Bot, update: BotUpdate) -> bool:
        if update.message is not None:
            message = update.message
        elif update.edited_message is not None:
            message = update.edited_message
        elif update.channel_post is not None:
            message = update.channel_post
        elif update.edited_channel_post is not None:
            message = update.edited_channel_post
        else:
            return False
        for content_type in self.content_types:
            if getattr(message, content_type.value) is not None:
                return True
        return False


@attr.s(frozen=True, auto_attribs=True)
class MessageTextFilter:
    pattern: 're.Pattern[str]'

    async def check(self, _: Bot, update: BotUpdate) -> bool:
        return (update.message is not None and
                update.message.text is not None and
                self.pattern.match(update.message.text) is not None)


@attr.s(frozen=True, auto_attribs=True)
class CallbackQueryDataFilter:
    pattern: 're.Pattern[str]'

    async def check(self, _: Bot, update: BotUpdate) -> bool:
        return (update.callback_query is not None and
                update.callback_query.data is not None and
                self.pattern.match(update.callback_query.data) is not None)


@attr.s(frozen=True)
class PrivateChatFilter:

    async def check(self, _: Bot, update: BotUpdate) -> bool:  # noqa
        return (update.message is not None and
                update.message.chat is not None and
                update.message.chat.type == ChatType.PRIVATE)


@attr.s(frozen=True)
class GroupChatFilter:

    async def check(self, _: Bot, update: BotUpdate) -> bool:  # noqa
        group_types = (ChatType.GROUP, ChatType.SUPERGROUP)
        return (update.message is not None and
                update.message.chat is not None and
                update.message.chat.type in group_types)


class ORFilter:

    def __init__(self, *filters: FilterProtocol) -> None:
        self._filters: Final[Tuple[FilterProtocol, ...]] = filters

    async def check(self, bot: Bot, update: BotUpdate) -> bool:
        return any([await _filter.check(bot, update)
                    for _filter in self._filters])


class ANDFilter:

    def __init__(self, *filters: FilterProtocol) -> None:
        self._filters: Final[Tuple[FilterProtocol, ...]] = filters

    async def check(self, bot: Bot, update: BotUpdate) -> bool:
        return all([await _filter.check(bot, update)
                    for _filter in self._filters])



import re
from pathlib import Path

from setuptools import setup  # type: ignore

path = Path(__file__).parent
txt = (path / 'aiotgbot' / '__init__.py').read_text('utf-8')
version = re.findall(r"^__version__ = '([^']+)'\r?$", txt, re.M)[0]
readme = (path / 'README.rst').read_text('utf-8')

setup(
    name='aiotgbot',
    version=version,
    description='Asynchronous library for Telegram bot API',
    long_description=readme,
    long_description_content_type="text/x-rst",
    url='https://github.com/gleb-chipiga/aiotgbot',
    license='MIT',
    author='Gleb Chipiga',
    # author_email='',
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'License :: OSI Approved :: MIT License',
        'Development Status :: 3 - Alpha',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Topic :: Internet',
        'Topic :: Communications :: Chat',
        'Framework :: AsyncIO',
    ],
    packages=['aiotgbot'],
    package_data={'aiotgbot': ['py.typed']},
    python_requires='>=3.8,<3.10',
    install_requires=['aiohttp', 'aiojobs', 'aiojobs-stubs>=0.3.0.post1',
                      'attrs', 'backoff', 'frozenlist', 'aiofreqlimit',
                      'yarl'],
    tests_require=['pytest', 'pytest-asyncio', 'hypothesis'],
    extras_require={'sqlite': ['aiosqlite'], 'passport': ['cryptography']}
)
